// Copyright (c) 2015, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'dart:core' hide Resource;

import 'package:analyzer/file_system/file_system.dart';
import 'package:analyzer/file_system/physical_file_system.dart';
import 'package:analyzer/source/package_map_resolver.dart';
import 'package:analyzer/src/generated/engine.dart';
import 'package:analyzer/src/generated/error.dart';
import 'package:analyzer/src/generated/java_io.dart';
import 'package:analyzer/src/generated/sdk.dart';
import 'package:analyzer/src/generated/sdk_io.dart';
import 'package:analyzer/src/generated/source.dart';
import 'package:analyzer/src/generated/source_io.dart';
import 'package:analyzer/src/plugin/engine_plugin.dart';
import 'package:plugin/manager.dart';

main() {
  ExtensionManager extensionManager = new ExtensionManager();
  EnginePlugin plugin = AnalysisEngine.instance.enginePlugin;
  if (plugin.taskExtensionPoint == null) {
    extensionManager.processPlugins([plugin]);
  }

  ResourceProvider resourceProvider = PhysicalResourceProvider.INSTANCE;

  DartSdk sdk = new DirectoryBasedDartSdk(
      new JavaFile('/Users/scheglov/Applications/dart-sdk'));
  AnalysisEngine.instance.useTaskModel = true;
  AnalysisContext context = AnalysisEngine.instance.createAnalysisContext();

  {
    var newOptions = new AnalysisOptionsImpl.from(context.analysisOptions);
    newOptions.hint = true;
    context.analysisOptions = newOptions;
  }

  var sourceFactory = new SourceFactory(<UriResolver>[
    new DartUriResolver(sdk),
    new PackageMapUriResolver(resourceProvider, <String, List<Folder>>{
      'analyzer': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/analyzer')
      ],
      'path': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/path')
      ],
      'plugin': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/plugin')
      ],
      'stack_trace': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/stack_trace')
      ],
      'typed_mock': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/typed_mock')
      ],
      'unittest': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/unittest')
      ],
      'watcher': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/watcher')
      ]
    }),
    new ResourceUriResolver(resourceProvider)
  ]);
//  context = createAnalysisContext();
  context.sourceFactory = sourceFactory;

  List<Source> sources = <Source>[];
  _addSources(
      sources,
      resourceProvider
          .getResource('/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer'));
//  print(sources.join('\n'));

//  Source source = new TestSource('/test.dart');
  Source source = sources.firstWhere((s) => s.fullName ==
      '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/lib/src/generated/error.dart');
  print(source);
  print(source.exists());

  ChangeSet changeSet = new ChangeSet();
  sources.forEach(changeSet.addedSource);
  context.applyChanges(changeSet);

  while (true) {
    AnalysisResult analysisResult = context.performAnalysisTask();
//    print(analysisResult.changeNotices);
    if (analysisResult.changeNotices == null) {
      break;
    }
    for (var n in analysisResult.changeNotices) {
      String fullName = n.source.fullName;
      if (fullName.startsWith(
              '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/') &&
          !fullName.contains('/packages/')) {
        List<AnalysisError> errors = n.errors;
        if (errors.isNotEmpty) {
          print(errors);
        }
      }
    }
  }

//  print('===============================================');
//  String contents = source.contents.data + ' class X {}';
////  print(contents);
//  context.setContents(source, contents);
//
//  print('===============================================');
//  print('===============================================');
//  print('===============================================');
//  print('===============================================');
//  while (true) {
//    AnalysisResult analysisResult = context.performAnalysisTask();
////    print(analysisResult.changeNotices);
//    if (analysisResult.changeNotices == null) {
//      break;
//    }
//    for (var n in analysisResult.changeNotices) {
//      String fullName = n.source.fullName;
//      if (fullName.startsWith(
//              '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/') &&
//          !fullName.contains('/packages/')) {
//        List<AnalysisError> errors = n.errors;
//        if (errors.isNotEmpty) {
//          print(errors);
//        }
//      }
//    }
//  }

//  context.setContents(source, r'''
//main() {
//  var vvv = 42;
//  print(vvv);
//}
//''');
//  CompilationUnit unit = context.resolveCompilationUnit2(source, source);
//  print(unit);
//  print(unit);
}

void _addSources(List<Source> sources, Resource resource) {
  if (resource is File) {
    sources.add(resource.createSource());
  }
  if (resource is Folder) {
    if (resource.shortName == 'packages') {
      return;
    }
    resource.getChildren().forEach((r) => _addSources(sources, r));
  }
}
