// Copyright (c) 2015, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'package:analyzer/src/generated/sdk_io.dart';
import 'package:analyzer/src/generated/sdk.dart';
import 'package:analyzer/src/generated/java_io.dart';
import 'package:analyzer/src/generated/engine.dart';
import 'package:analyzer/src/generated/source.dart';
import 'package:plugin/manager.dart';
import 'package:analyzer/src/plugin/engine_plugin.dart';
import 'package:analyzer/file_system/file_system.dart';
import 'package:analyzer/file_system/physical_file_system.dart';
import 'package:analyzer/source/package_map_resolver.dart';
import 'package:analyzer/src/generated/element.dart';

InternalAnalysisContext context;
List<Source> sources = <Source>[];
final List<AnalysisErrorInfo> errorInfos = new List<AnalysisErrorInfo>();

main() {
  ExtensionManager extensionManager = new ExtensionManager();
  EnginePlugin plugin = AnalysisEngine.instance.enginePlugin;
  if (plugin.taskExtensionPoint == null) {
    extensionManager.processPlugins([plugin]);
  }

  ResourceProvider resourceProvider = PhysicalResourceProvider.INSTANCE;

  DartSdk sdk = new DirectoryBasedDartSdk(
      new JavaFile('/Users/scheglov/Applications/dart-sdk'));
  AnalysisEngine.instance.useTaskModel = true;
  context = AnalysisEngine.instance.createAnalysisContext();
  var sourceFactory = new SourceFactory(<UriResolver>[
    new DartUriResolver(sdk),
    new PackageMapUriResolver(resourceProvider, <String, List<Folder>>{
      'analyzer': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/analyzer')
      ],
      'analysis_server': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analysis_server/packages/analysis_server')
      ],
      'collection': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/collection')
      ],
      'logging': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/logging')
      ],
      'path': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/path')
      ],
      'plugin': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/plugin')
      ],
      'stack_trace': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/stack_trace')
      ],
      'typed_mock': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/typed_mock')
      ],
      'unittest': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/unittest')
      ],
      'watcher': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/watcher')
      ]
    }),
    new ResourceUriResolver(resourceProvider)
  ]);
//  context = createAnalysisContext();
  context.sourceFactory = sourceFactory;

  File file = resourceProvider.getResource(
      '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analysis_server/test/operation/operation_test.dart');
  Source source = file.createSource();
  // ERRORS
  LibraryElement libraryElement = context.computeLibraryElement(source);
  prepareSources(libraryElement);
  prepareErrors();
  print(errorInfos.join('\n'));
  // RESOLVE
//  CompilationUnit unit = context.resolveCompilationUnit2(source, source);
//  print(unit);
//  print(unit);
//  {
//    var changeSet = new ChangeSet()..changedSource(source);
//    context.applyChanges(changeSet);
//  }
//  var cache = context.analysisCache;
//  print(cache);
//  print(cache);
}

void prepareSources(LibraryElement library) {
  var units = new Set<CompilationUnitElement>();
  var libraries = new Set<LibraryElement>();
  addLibrarySources(library, libraries, units);
}

void addLibrarySources(LibraryElement library, Set<LibraryElement> libraries,
    Set<CompilationUnitElement> units) {
  if (library == null || !libraries.add(library)) {
    return;
  }
  // may be skip library
  {
    UriKind uriKind = library.source.uriKind;
    // Optionally skip package: libraries.
    if (/*!options.showPackageWarnings &&*/ _isOtherPackage(
        library.source.uri)) {
      return;
    }
    // Optionally skip SDK libraries.
    if (/*!options.showSdkWarnings &&*/ uriKind == UriKind.DART_URI) {
      return;
    }
  }
  // add compilation units
  addCompilationUnitSource(library.definingCompilationUnit, libraries, units);
  for (CompilationUnitElement child in library.parts) {
    addCompilationUnitSource(child, libraries, units);
  }
  // add referenced libraries
  for (LibraryElement child in library.importedLibraries) {
    addLibrarySources(child, libraries, units);
  }
  for (LibraryElement child in library.exportedLibraries) {
    addLibrarySources(child, libraries, units);
  }
}

void addCompilationUnitSource(CompilationUnitElement unit,
    Set<LibraryElement> libraries, Set<CompilationUnitElement> units) {
  if (unit == null || units.contains(unit)) {
    return;
  }
  units.add(unit);
  sources.add(unit.source);
}

/// Determine whether the given URI refers to a package other than the package
/// being analyzed.
bool _isOtherPackage(Uri uri) {
  if (uri.scheme != 'package') {
    return false;
  }
  if ('' != null &&
      uri.pathSegments.length > 0 &&
      uri.pathSegments[0] == 'analysis_server') {
    return false;
  }
  return true;
}

/// Fills [errorInfos] using [sources].
void prepareErrors() {
  for (Source source in sources) {
    print(source);
    context.computeErrors(source);
    var sourceErrors = context.getErrors(source);
    errorInfos.add(sourceErrors);
  }
}
