// Copyright (c) 2015, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'dart:core' hide Resource;

import 'package:analyzer/src/generated/sdk_io.dart';
import 'package:analyzer/src/generated/sdk.dart';
import 'package:analyzer/src/generated/java_io.dart';
import 'package:analyzer/src/generated/engine.dart' hide AnalysisCache;
import 'package:analyzer/src/generated/source.dart';
import 'package:plugin/manager.dart';
import 'package:analyzer/src/plugin/engine_plugin.dart';
import 'package:analyzer/file_system/file_system.dart';
import 'package:analyzer/file_system/physical_file_system.dart';
import 'package:analyzer/source/package_map_resolver.dart';
import 'dart:async';

InternalAnalysisContext context;
List<Source> sources = <Source>[];
final List<AnalysisErrorInfo> errorInfos = new List<AnalysisErrorInfo>();

main() {
  ExtensionManager extensionManager = new ExtensionManager();
  EnginePlugin plugin = AnalysisEngine.instance.enginePlugin;
  if (plugin.taskExtensionPoint == null) {
    extensionManager.processPlugins([plugin]);
  }

  ResourceProvider resourceProvider = PhysicalResourceProvider.INSTANCE;

  DartSdk sdk = new DirectoryBasedDartSdk(
      new JavaFile('/Users/scheglov/Applications/dart-sdk'));
  AnalysisEngine.instance.useTaskModel = true;
  AnalysisEngine.instance.limitInvalidationInTaskModel = true;
  context = AnalysisEngine.instance.createAnalysisContext();
  var sourceFactory = new SourceFactory(<UriResolver>[
    new DartUriResolver(sdk),
    new PackageMapUriResolver(resourceProvider, <String, List<Folder>>{
      'analyzer': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/analyzer')
      ],
      'analysis_server': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analysis_server/packages/analysis_server')
      ],
      'collection': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/collection')
      ],
      'logging': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/logging')
      ],
      'path': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/path')
      ],
      'plugin': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/plugin')
      ],
      'stack_trace': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/stack_trace')
      ],
      'typed_mock': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/typed_mock')
      ],
      'unittest': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/unittest')
      ],
      'watcher': [
        resourceProvider.getResource(
            '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analyzer/packages/watcher')
      ]
    }),
    new ResourceUriResolver(resourceProvider)
  ]);
//  context = createAnalysisContext();
  context.sourceFactory = sourceFactory;

  List<Source> sources = <Source>[];
  addSources(
      sources, resourceProvider.getResource('/Users/scheglov/dart/path/lib'));
  print(sources.join('\n'));

//  File file = resourceProvider.getResource(
//      '/Users/scheglov/Source/Dart/sdk.git/sdk/pkg/analysis_server/test/operation/operation_test.dart');
//  Source source = file.createSource();

  {
    ChangeSet changeSet = new ChangeSet();
    for (Source source in sources) {
      changeSet.addedSource(source);
    }
    context.applyChanges(changeSet);
  }

  doFullAnalysis();

  {
    Stopwatch sw = new Stopwatch()..start();
    for (Source source in sources) {
      if (source.fullName ==
          '/Users/scheglov/dart/path/lib/src/characters.dart') {
        String contents = source.contents.data;
        String newContents = contents.replaceAll('ZZZ {}', 'ZZZ2 {}');
        context.setContents(source, newContents);
        break;
      }
    }
    print('invalidate time: ${sw.elapsedMilliseconds}');
  }

  {
    Stopwatch sw = new Stopwatch()..start();
    doFullAnalysis();
    print('alanlysis-2 time: ${sw.elapsedMilliseconds}');
  }

  sleepForever();
}

void doFullAnalysis() {
  while (true) {
    AnalysisResult analysisResult = context.performAnalysisTask();
    if (!analysisResult.hasMoreWork) {
      break;
    }
  }
}

void sleepForever() {
  new Future.delayed(new Duration(seconds: 1), sleepForever);
}

void addSources(List<Source> sources, Folder folder) {
  for (Resource resource in folder.getChildren()) {
    if (resource is File) {
      sources.add(resource.createSource());
    } else {
      addSources(sources, resource);
    }
  }
}
